# general purpose 'tooltip' routines - currently unused in idlefork
# (although the 'calltips' extension is partly based on this code)
# may be useful for some purposes in (or almost in ;) the current project scope
# Ideas gleaned from PySol

from Tkinter import *
from colorsys import rgb_to_hls

COLOR_TABLE = {
    'black':   (0.0, 0.0, 0.0),
    'white':   (1.0, 1.0, 1.0),
    'red':     (1.0, 0.0, 0.0),
    'green':   (0.0, 1.0, 0.0),
    'blue':    (0.0, 0.0, 1.0),
    'cyan':    (0.0, 1.0, 1.0),
    'yellow':  (1.0, 1.0, 0.0),
    'magenta': (1.0, 0.0, 1.0),
    }

def parse_color(color):
    if color.startswith('#'):
        n = len(color) - 1
        if n % 3 != 0:
            raise ValueError('illegal color value: ' + repr(color))
        partlen = n // 3
        maxval = 0xf
        for i in xrange(partlen-1):
            maxval = (maxval << 4) + 0xf
        maxval = float(maxval)

        i, j = 1, 1 + partlen
        r = int(color[i:j], 16)
        i, j = j, j + partlen
        g = int(color[i:j], 16)
        i, j = j, j + partlen
        b = int(color[i:j], 16)
        return r / maxval, g / maxval, b / maxval
    else:
        try:
            return COLOR_TABLE[color.lower()]
        except KeyError:
            raise ValueError('unknown color name: ' + repr(color))

def set_tooltip_bg(widget):
    r, g, b = parse_color(widget['fg'])
    h, l, s = rgb_to_hls(r, g, b)
    if l > 0.5:
        widget['bg'] = "#201F1F"
    else:
        widget['bg'] = "#ffffe0"

class ToolTipBase:

    def __init__(self, button):
        self.button = button
        self.tipwindow = None
        self.id = None
        self.x = self.y = 0
        self._id1 = self.button.bind("<Enter>", self.enter)
        self._id2 = self.button.bind("<Leave>", self.leave)
        self._id3 = self.button.bind("<ButtonPress>", self.leave)

    def enter(self, event=None):
        self.schedule()

    def leave(self, event=None):
        self.unschedule()
        self.hidetip()

    def schedule(self):
        self.unschedule()
        self.id = self.button.after(1500, self.showtip)

    def unschedule(self):
        id = self.id
        self.id = None
        if id:
            self.button.after_cancel(id)

    def showtip(self):
        if self.tipwindow:
            return
        # The tip window must be completely outside the button;
        # otherwise when the mouse enters the tip window we get
        # a leave event and it disappears, and then we get an enter
        # event and it reappears, and so on forever :-(
        x = self.button.winfo_rootx() + 20
        y = self.button.winfo_rooty() + self.button.winfo_height() + 1
        self.tipwindow = tw = Toplevel(self.button)
        tw.wm_overrideredirect(1)
        tw.wm_geometry("+%d+%d" % (x, y))
        self.showcontents()

    def showcontents(self, text="Your text here"):
        # Override this in derived class
        label = Label(self.tipwindow, text=text, justify=LEFT,
                      relief=SOLID, borderwidth=1)
        set_tooltip_bg(label)
        label.pack()

    def hidetip(self):
        tw = self.tipwindow
        self.tipwindow = None
        if tw:
            tw.destroy()

class ToolTip(ToolTipBase):
    def __init__(self, button, text):
        ToolTipBase.__init__(self, button)
        self.text = text
    def showcontents(self):
        ToolTipBase.showcontents(self, self.text)

class ListboxToolTip(ToolTipBase):
    def __init__(self, button, items):
        ToolTipBase.__init__(self, button)
        self.items = items
    def showcontents(self):
        listbox = Listbox(self.tipwindow)
        set_tooltip_bg(listbox)
        listbox.pack()
        for item in self.items:
            listbox.insert(END, item)

def main():
    # Test code
    root = Tk()
    b = Button(root, text="Hello", command=root.destroy)
    b.pack()
    root.update()
    tip = ListboxToolTip(b, ["Hello", "world"])
    root.mainloop()

if __name__ == '__main__':
    main()
